package ch.psi.daq.queryrest.controller.validator;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.Set;

import javax.annotation.Resource;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQuery;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.operation.Aggregation;
import ch.psi.daq.domain.query.operation.QueryField;
import ch.psi.daq.queryrest.config.QueryRestConfig;

public class QueryValidator implements Validator {

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_FIELDS)
   private Set<QueryField> defaultResponseFields;

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_AGGREGATIONS)
   private Set<Aggregation> defaultResponseAggregations;

   /**
    * {@inheritDoc}
    */
   @Override
   public boolean supports(Class<?> clazz) {
      return DAQQuery.class.isAssignableFrom(clazz) || DAQQueries.class.isAssignableFrom(clazz);
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public void validate(Object target, Errors errors) {
      if (target instanceof DAQQuery) {
         this.checkElement((DAQQuery) target);
      }else if(target instanceof DAQQueries){
         DAQQueries queries = (DAQQueries) target;
         for (DAQQueryElement daqQueryElement : queries) {
            this.checkElement(daqQueryElement);
         }
      }
   }

   private void checkElement(DAQQueryElement query) {
      // set default values (if not set)
      if (query.getFields() == null || query.getFields().isEmpty()) {
         query.setFields(new LinkedHashSet<>(defaultResponseFields));
      }

      if (query.getAggregations() == null || query.getAggregations().isEmpty()) {
         query.setAggregations(new ArrayList<>(defaultResponseAggregations));
      }
   }
}
