package ch.psi.daq.test.queryrest;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurationSupport;

import ch.psi.daq.cassandra.reader.CassandraReader;
import ch.psi.daq.domain.reader.DataReader;
import ch.psi.daq.domain.status.StatusReader;
import ch.psi.daq.query.config.QueryConfig;
import ch.psi.daq.query.processor.QueryProcessor;
import ch.psi.daq.query.processor.QueryProcessorLocal;
import ch.psi.daq.test.query.config.LocalQueryTestConfig;
import ch.psi.daq.test.queryrest.query.DummyArchiverApplianceReader;
import ch.psi.daq.test.queryrest.query.DummyCassandraReader;
import ch.psi.daq.test.queryrest.status.DummyArchiverApplianceStatusReader;
import ch.psi.daq.test.queryrest.status.DummyCassandraStatusReader;

@Configuration
@ComponentScan
@EnableWebMvc
@PropertySources(value = {
      @PropertySource(value = {"classpath:queryrest-test.properties"})
})
public class DaqWebMvcConfig extends WebMvcConfigurationSupport {

   // ensure that properties in dispatcher.properties are loaded first and then overwritten by the
   // properties in dispatcher-test.properties
   @Import(value = {LocalQueryTestConfig.class})
   static class InnerConfiguration {
   }

   @Bean(name = QueryConfig.BEAN_NAME_CASSANDRA_QUERY_PROCESSOR)
   @Lazy
   public QueryProcessor cassandraQueryProcessor() {
      return new QueryProcessorLocal(cassandraReader());
   }

   @Bean
   @Lazy
   public CassandraReader cassandraReader() {
      return new DummyCassandraReader();
   }

   @Bean
   @Lazy
   public DataReader archiverApplianceReader() {
      return new DummyArchiverApplianceReader();
   }

   @Bean(name = QueryConfig.BEAN_NAME_CASSANDRA_STATUS_READER)
   @Lazy
   public StatusReader cassandraStatusReader() {
      return new DummyCassandraStatusReader();
   }

   @Bean(name = QueryConfig.BEAN_NAME_ARCHIVER_APPLIANCE_STATUS_READER)
   @Lazy
   public StatusReader archiverApplianceStatusReader() {
      return new DummyArchiverApplianceStatusReader();
   }
}
