package ch.psi.daq.test.queryrest.controller;

import org.junit.After;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.common.time.TimeUtils;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQuery;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.channels.ChannelsRequest;
import ch.psi.daq.domain.query.operation.AggregationType;
import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.operation.QueryField;
import ch.psi.daq.domain.reader.Backend;
import ch.psi.daq.domain.request.range.RequestRangeDate;
import ch.psi.daq.domain.request.range.RequestRangePulseId;
import ch.psi.daq.domain.request.range.RequestRangeTime;
import ch.psi.daq.domain.test.TestTimeUtils;
import ch.psi.daq.queryrest.controller.QueryRestController;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public class QueryRestControllerJsonTest extends AbstractDaqRestTest {

   public static final String TEST_CHANNEL_01 = "testChannel1";
   public static final String TEST_CHANNEL_02 = "testChannel2";
   public static final String[] TEST_CHANNEL_NAMES = new String[] {TEST_CHANNEL_01, TEST_CHANNEL_02};

   @After
   public void tearDown() throws Exception {}

   @Test
   public void testChannelNameQuery() throws Exception {

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .get(QueryRestController.PATH_CHANNELS)
                        .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").value("BoolScalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").value("BoolWaveform"))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].backend").value(Backend.archiverappliance.name()))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels").isArray())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[0]").exists())
      // .andExpect(
      // MockMvcResultMatchers.jsonPath("$[1].channels[0]").value(DummyArchiverApplianceReader.TEST_CHANNEL_1))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[1]").exists())
      // .andExpect(
      // MockMvcResultMatchers.jsonPath("$[1].channels[1]").value(DummyArchiverApplianceReader.TEST_CHANNEL_2))
      ;

   }

   @Test
   public void testSpecificChannelSearch() throws Exception {
      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .get(QueryRestController.PATH_CHANNELS + "/int32")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").value("Int32Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").value("Int32Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").value("UInt32Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").value("UInt32Waveform"))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].backend").value(Backend.archiverappliance.name()))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels").isArray())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[0]").doesNotExist())
      ;
   }

   @Test
   public void testChannelNameQueryBackendOrder() throws Exception {
      ChannelsRequest request = new ChannelsRequest("int64", Ordering.desc, Backend.SF_DATABUFFER);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[0]").value("UInt64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[1]").value("UInt64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[2]").value("Int64Waveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[3]").value("Int64Scalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").doesNotExist());
   }

   @Test
   public void testChannelNameQueryReload() throws Exception {
      ChannelsRequest request = new ChannelsRequest();

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[23]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[24]").doesNotExist())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].backend").value(Backend.archiverappliance.name()))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels").isArray())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[2]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[3]").doesNotExist())
      ;

      // each reload add another channel
      request.setReload(true);

      content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[24]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channels[25]").doesNotExist())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].backend").value(Backend.archiverappliance.name()))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels").isArray())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[3]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channels[4]").doesNotExist())
      ;
   }

   @Test
   public void testCorsFilterNoHeaders() throws Exception {
      this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).build();

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.PATH_CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            // we didn't set the 'Origin' header so no access-control
            .andExpect(MockMvcResultMatchers.header().doesNotExist("Access-Control-Allow-Origin"));
   }

   @Test
   public void testCorsFilterIncludesHeaders() throws Exception {
      // all headers are set
      this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).build(); 
      
      // curl -H "Origin: *" -H "Access-Control-Request-Method: POST" -X OPTIONS -v http://localhost:8080/channels
      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.PATH_CHANNELS)
                  .header("Origin", "*")
                  .header("Access-Control-Request-Method", "POST")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "*"));

      // curl -H "Origin: http://localhost:8080" -H "Access-Control-Request-Method: POST" -X OPTIONS -v http://localhost:8080/channels
      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.PATH_CHANNELS)
                  .header("Origin", "http://localhost:8080")
                  .header("Access-Control-Request-Method", "POST")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "http://localhost:8080"));
   }

   @Test
   public void testPulseRangeQuery() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  100,
                  101),
            TEST_CHANNEL_NAMES);
      request.addField(QueryField.pulseId);
      request.addField(QueryField.globalSeconds);
      request.addField(QueryField.globalMillis);
      request.addField(QueryField.iocSeconds);
      request.addField(QueryField.iocMillis);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalMillis").value(1000))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].iocSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].iocMillis").value(1000))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalMillis").value(1010))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].iocSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].iocMillis").value(1010))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.name").value(TEST_CHANNEL_02))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].globalMillis").value(1000))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].iocSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].iocMillis").value(1000))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].globalMillis").value(1010))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].iocSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].iocMillis").value(1010));
   }
   
   @Test
   public void testPulseRangeQuery_Fields() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  100,
                  199),
            TEST_CHANNEL_NAMES);
      request.addField(QueryField.pulseId);
      request.addField(QueryField.eventCount);
      request.setNrOfBins(2);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].eventCount").value(50))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(150))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].eventCount").value(50))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.name").value(TEST_CHANNEL_02))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].eventCount").value(50))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(150))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].eventCount").value(50));
   }

   @Test
   public void testPulseRangeQueryBackends() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  100,
                  101),
            new ChannelName(TEST_CHANNEL_01, Backend.SF_DATABUFFER),
            new ChannelName(TEST_CHANNEL_02, Backend.SF_ARCHIVERAPPLIANCE));

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").isMap())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.name").value(TEST_CHANNEL_02))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(10))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].globalMillis").value(100))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(11))
      // .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].globalMillis").value(110))
      ;
   }

   @Test
   public void testPulseRangeQueries() throws Exception {
      String testChannel3 = "testChannel3";
      DAQQueries request = new DAQQueries(
            new DAQQueryElement(
                  new RequestRangePulseId(
                        100,
                        101),
                  TEST_CHANNEL_NAMES),
            new DAQQueryElement(
                  new RequestRangePulseId(
                        100,
                        101),
                  testChannel3));

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERIES)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].channel.name").value(TEST_CHANNEL_02))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0][1].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0].channel.name").value(testChannel3))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1][0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)));
   }

   @Test
   public void testTimeRangeQuery() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangeTime(
                  TimeUtils.getTimeFromMillis(2000, 0),
                  TimeUtils.getTimeFromMillis(2010, 0)),
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc.perform(MockMvcRequestBuilders
            .post(QueryRestController.PATH_QUERY)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(200))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(2, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(201))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(2, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.name").value(TEST_CHANNEL_02))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(200))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(2, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(201))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(2, 10000000)));
   }

   @Test
   public void testDateRangeQuery() throws Exception {
      String startDate = TimeUtils.format(1000);
      String endDate = TimeUtils.format(1010);
      DAQQuery request = new DAQQuery(
            new RequestRangeDate(
                  startDate,
                  endDate),
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .post(QueryRestController.PATH_QUERY)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(content)
            )
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.name").value(TEST_CHANNEL_02))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel.backend").value("sf-databuffer"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 10000000)));
   }

   @Test
   public void testExtremaAggregation() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  100,
                  101),
            Ordering.asc,
            AggregationType.extrema,
            TEST_CHANNEL_NAMES[0]);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())

            .andExpect(MockMvcResultMatchers.jsonPath("$").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_NAMES[0]))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.value").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.value").value(Double.valueOf(100)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event.pulseId").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event.pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.value").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.value").value(Double.valueOf(101)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event.pulseId").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event.pulseId").value(101));
   }

   @Test
   public void testDateRangeQueryNrOfBinsAggregate() throws Exception {
      long startTime = 1000;
      long endTime = 1099;
      String startDate = TimeUtils.format(startTime);
      String endDate = TimeUtils.format(endTime);
      DAQQuery request = new DAQQuery(
            new RequestRangeDate(
                  startDate,
                  endDate),
            TEST_CHANNEL_01);
      request.setNrOfBins(2);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .post(QueryRestController.PATH_QUERY)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(content)
            )
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].eventCount").value(5))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(105))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(1, 50000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].eventCount").value(5));
   }

   @Test
   public void testDateRangeQueryBinSizeAggregate() throws Exception {
      long startTime = 10000;
      long endTime = 10999;
      String startDate = TimeUtils.format(startTime);
      String endDate = TimeUtils.format(endTime);
      DAQQuery request = new DAQQuery(
            new RequestRangeDate(
                  startDate,
                  endDate),
            TEST_CHANNEL_01);
      request.setBinSize(100);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .post(QueryRestController.PATH_QUERY)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(content)
            )
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").isMap())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.name").value(TEST_CHANNEL_01))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel.backend").value(Backend.SF_DATABUFFER.getKey()))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(1000))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 0)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(1010))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 100000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[2].pulseId").value(1020))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[2].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 200000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[2].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[3].pulseId").value(1030))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[3].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 300000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[3].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[4].pulseId").value(1040))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[4].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 400000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[4].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[5].pulseId").value(1050))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[5].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 500000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[5].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[6].pulseId").value(1060))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[6].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 600000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[6].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[7].pulseId").value(1070))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[7].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 700000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[7].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[8].pulseId").value(1080))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[8].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 800000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[8].eventCount").value(10))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[9].pulseId").value(1090))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[9].globalSeconds").value(
                  TestTimeUtils.getTimeStr(10, 900000000)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[9].eventCount").value(10));
   }

   @Test
   public void testGzipFileSuffixHeader() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  10,
                  11),
            TEST_CHANNEL_NAMES);
      request.setCompression(Compression.GZIP);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Content-Disposition", "attachment; filename=data.gz"));
   }

   @Test
   public void testJsonFileSuffixHeader() throws Exception {
      DAQQuery request = new DAQQuery(
            new RequestRangePulseId(
                  10,
                  11),
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.PATH_QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Content-Disposition", "attachment; filename=data.json"));
   }
}
