package ch.psi.daq.test.queryrest.query;

import java.util.List;
import java.util.concurrent.atomic.AtomicLong;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import com.google.common.collect.Lists;

import ch.psi.daq.domain.StreamEvent;
import ch.psi.daq.domain.query.backend.PulseIdRangeQuery;
import ch.psi.daq.domain.query.backend.TimeRangeQuery;
import ch.psi.daq.domain.query.event.EventQuery;
import ch.psi.daq.domain.reader.Backend;
import ch.psi.daq.domain.reader.DataReader;

public class DummyArchiverApplianceReader implements DataReader {
   public static final String ARCHIVER_TEST_CHANNEL = "ArchiverTestChannel_";

   public static final String TEST_CHANNEL_1 = "ArchiverChannel_1";
   public static final String TEST_CHANNEL_2 = "ArchiverChannel_2";
   private List<String> channels = Lists.newArrayList(TEST_CHANNEL_1, TEST_CHANNEL_2);

   private AtomicLong channelNameCallCounter = new AtomicLong();


   @Override
   public Backend getBackend() {
      return Backend.SF_ARCHIVERAPPLIANCE;
   }

   @Override
   public Stream<String> getChannelStream(String regex) {
      channels.add(ARCHIVER_TEST_CHANNEL + channelNameCallCounter.incrementAndGet());

      Stream<String> channelStream = channels.stream();
      if (regex != null) {
         Pattern pattern = Pattern.compile(regex);
         channelStream = channelStream.filter(channel -> pattern.matcher(channel).find());
      }

      return channelStream;
   }

   @Override
   public Stream<? extends StreamEvent> getEventStream(PulseIdRangeQuery query) {
      return DummyCassandraReader.getDummyEventStream(query.getChannel(), query.getStartPulseId(), query.getEndPulseId(),
            query.getEventColumns())
            .filter(query.getFilterOrDefault(EventQuery.NO_OP_FILTER));
   }

   @Override
   public Stream<? extends StreamEvent> getEventStream(TimeRangeQuery query) {
      return DummyCassandraReader.getDummyEventStream(query.getChannel(), query.getStartMillis() / 10, query.getEndMillis() / 10)
            .filter(query.getFilterOrDefault(EventQuery.NO_OP_FILTER));
   }
}
